package com.sun.jmx.mbeanserver;

import java.util.Iterator;
import javax.management.InvalidAttributeValueException;
import javax.management.Attribute;
import javax.management.AttributeList;
import javax.management.ReflectionException;
import javax.management.MBeanException;
import javax.management.AttributeNotFoundException;
import javax.management.ObjectName;
import javax.management.MBeanServer;
import javax.management.NotCompliantMBeanException;
import sun.reflect.misc.ReflectUtil;
import javax.management.MBeanInfo;
import javax.management.MBeanRegistration;

public abstract class MBeanSupport<M> implements DynamicMBean2, MBeanRegistration
{
    private final MBeanInfo mbeanInfo;
    private final Object resource;
    private final PerInterface<M> perInterface;
    
     <T> MBeanSupport(T resource, Class<T> clazz) throws NotCompliantMBeanException {
        if (clazz == null) {
            throw new NotCompliantMBeanException("Null MBean interface");
        }
        if (!clazz.isInstance(resource)) {
            throw new NotCompliantMBeanException("Resource class " + resource.getClass().getName() + " is not an instance of " + clazz.getName());
        }
        ReflectUtil.checkPackageAccess(clazz);
        this.resource = resource;
        MBeanIntrospector<M> mBeanIntrospector = this.getMBeanIntrospector();
        this.perInterface = mBeanIntrospector.getPerInterface(clazz);
        this.mbeanInfo = mBeanIntrospector.getMBeanInfo(resource, this.perInterface);
        return;
    }
    
    abstract MBeanIntrospector<M> getMBeanIntrospector();
    
    abstract Object getCookie();
    
    public final boolean isMXBean() {
        return this.perInterface.isMXBean();
    }
    
    public abstract void register(MBeanServer p0, ObjectName p1) throws Exception;
    
    public abstract void unregister();
    
    @Override
    public final ObjectName preRegister(MBeanServer mBeanServer, ObjectName preRegister) throws Exception {
        if ((this.resource instanceof MBeanRegistration)) {
            preRegister = ((MBeanRegistration)this.resource).preRegister(mBeanServer, preRegister);
            return preRegister;
        }
        return preRegister;
    }
    
    @Override
    public final void preRegister2(MBeanServer mBeanServer, ObjectName objectName) throws Exception {
        this.register(mBeanServer, objectName);
    }
    
    @Override
    public final void registerFailed() {
        this.unregister();
    }
    
    @Override
    public final void postRegister(Boolean b) {
        if ((this.resource instanceof MBeanRegistration)) {
            ((MBeanRegistration)this.resource).postRegister(b);
        }
    }
    
    @Override
    public final void preDeregister() throws Exception {
        if ((this.resource instanceof MBeanRegistration)) {
            ((MBeanRegistration)this.resource).preDeregister();
        }
    }
    
    @Override
    public final void postDeregister() {
        try {
            this.unregister();
        }
        finally {
            if ((this.resource instanceof MBeanRegistration)) {
                ((MBeanRegistration)this.resource).postDeregister();
            }
        }
    }
    
    @Override
    public final Object getAttribute(String s) throws AttributeNotFoundException, MBeanException, ReflectionException {
        return this.perInterface.getAttribute(this.resource, s, this.getCookie());
    }
    
    @Override
    public final AttributeList getAttributes(String[] array) {
        AttributeList list = new AttributeList(array.length);
        for (String name : array) {
            try {
                list.add(new Attribute(name, this.getAttribute(name)));
            }
            catch (Exception ex) {}
        }
        return list;
    }
    
    @Override
    public final void setAttribute(Attribute attribute) throws AttributeNotFoundException, InvalidAttributeValueException, MBeanException, ReflectionException {
        this.perInterface.setAttribute(this.resource, attribute.getName(), attribute.getValue(), this.getCookie());
    }
    
    @Override
    public final AttributeList setAttributes(AttributeList list) {
        AttributeList list2 = new AttributeList(list.size());
        for (Attribute attribute : list) {
            try {
                this.setAttribute(attribute);
                list2.add(new Attribute(attribute.getName(), attribute.getValue()));
            }
            catch (Exception ex) {}
        }
        return list2;
    }
    
    @Override
    public final Object invoke(String s, Object[] array, String[] array2) throws MBeanException, ReflectionException {
        return this.perInterface.invoke(this.resource, s, array, array2, this.getCookie());
    }
    
    @Override
    public MBeanInfo getMBeanInfo() {
        return this.mbeanInfo;
    }
    
    @Override
    public final String getClassName() {
        return this.resource.getClass().getName();
    }
    
    @Override
    public final Object getResource() {
        return this.resource;
    }
    
    public final Class<?> getMBeanInterface() {
        return this.perInterface.getMBeanInterface();
    }
}
